package org.eclipse.graphiti.examples.diagram;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EcoreFactory;
import org.eclipse.graphiti.features.ICreateConnectionFeature;
import org.eclipse.graphiti.features.IFeatureProvider;
import org.eclipse.graphiti.features.context.ICreateConnectionContext;
import org.eclipse.graphiti.features.context.impl.AddConnectionContext;
import org.eclipse.graphiti.features.impl.AbstractCreateConnectionFeature;
import org.eclipse.graphiti.mm.pictograms.Connection;
import org.eclipse.graphiti.mm.pictograms.PictogramElement;

public class CreateEReferenceFeature extends AbstractCreateConnectionFeature
		implements ICreateConnectionFeature {

	public CreateEReferenceFeature(IFeatureProvider fp) {
		super(fp, "EReference", "Creates a new EReference");
	}

	@Override
	public boolean canStartConnection(ICreateConnectionContext context) {
		return getBusinessObjectForPictogramElement(context.getSourcePictogramElement()) instanceof EClass;
	}

	@Override
	public boolean canCreate(ICreateConnectionContext context) {
		PictogramElement sourcePictogramElement = context.getSourcePictogramElement();
		PictogramElement targetPictogramElement = context.getTargetPictogramElement();
		if (getBusinessObjectForPictogramElement(sourcePictogramElement) instanceof EClass && getBusinessObjectForPictogramElement(targetPictogramElement) instanceof EClass) {
			return true;
		}
		return false;
	}

	@Override
	public Connection create(ICreateConnectionContext context) {
		Connection newConnection = null;

		EClass source = (EClass) getBusinessObjectForPictogramElement(context.getSourcePictogramElement());
		EClass target = (EClass) getBusinessObjectForPictogramElement(context.getTargetPictogramElement());

		EReference eReference = EcoreFactory.eINSTANCE.createEReference();
		eReference.setName("new EReference");
		eReference.setEType(target);
		eReference.setLowerBound(0);
		eReference.setUpperBound(1);
		source.getEStructuralFeatures().add(eReference);

		AddConnectionContext addContext = new AddConnectionContext(context.getSourceAnchor(), context.getTargetAnchor());
		addContext.setNewObject(eReference);
		newConnection = (Connection) getFeatureProvider().addIfPossible(addContext);

		return newConnection;
	}
}
